/*****************************************************************************
 *
 *                        CalC (Calcium Calculator)
 *                             Victor Matveev
 *
 *                                vector.h
 *
 *                   A basic linear algebra vector class 
 *
 ******************************************************************************/

#ifndef CALC_VectorObj_H_included
#define CALC_VectorObj_H_included

extern int VERBOSE;

class VectorObj {

 protected:

 public:

  double *elem;
  long size;

  VectorObj(long n = 0)  { elem = new double[size = n]; };

  ~VectorObj() { delete [] elem; }

  VectorObj(const VectorObj &v);

  void set_size(long n) { size = n; }
  long get_size()  { return size; };

  double  operator[](long index) const  { return elem[index]; };
  double& operator[](long index) { return elem[index]; };

  double *operator()() const  { return elem; };

  VectorObj& operator=(const double *e);
  VectorObj& operator=(const double val);
  VectorObj& operator=(const VectorObj &v);

  void import(const char *);
  void Export(const char *);

  friend VectorObj operator-(const VectorObj &v);
  friend VectorObj operator-(const VectorObj &v1, const VectorObj &v2);
  friend VectorObj operator-(const VectorObj &v1, const double val);
  friend VectorObj operator-(const double val, const VectorObj &v1);

  friend void operator-=(VectorObj &v1, const VectorObj &v2);
  friend void operator-=(VectorObj &v1, const double val);

  friend VectorObj operator+(const VectorObj &v1, const VectorObj &v2);
  friend VectorObj operator+(const VectorObj &v1, const double val);
  friend VectorObj operator+(const double val, const VectorObj &v1);

  friend void operator+=(VectorObj &v1, const VectorObj &v2);
  friend void operator+=(VectorObj &v1, const double val);

  friend double operator*(const VectorObj &v1, const VectorObj &v2);
  friend VectorObj operator%(const VectorObj &v1, const VectorObj &v2);

  friend VectorObj operator*(const VectorObj &v1, const double val);
  friend VectorObj operator*(const double val, const VectorObj &v1);
  friend VectorObj operator%=(VectorObj &v1, const VectorObj &v2);
  friend VectorObj operator*=(VectorObj &v1, const double val);

  double norm() const;
  double fourier111() const;
  double pnorm() const;
  double collapse() const;
  void   print(int n=4) const;
};

double norm_diff(VectorObj &, VectorObj &);

#endif
